@@ -1,8 +1,8 @@
 geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussian, corstr = "independence", Mv = 1, weights = NULL, corr.mat = NULL, init.beta=NULL, init.alpha=NULL, init.phi = 1, scale.fix=FALSE, nodummy=FALSE, sandwich=TRUE, maxit=20, tol=0.00001){
   call <- match.call()
-  
+
   famret <- getfam(family)
-  
+
   if(inherits(famret, "family")){
     LinkFun <- famret$linkfun
     InvLink <- famret$linkinv
@@ -13,16 +13,16 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     VarFun <- famret$VarFun
     InvLink <- famret$InvLink
     InvLinkDeriv <- famret$InvLinkDeriv
-  }  
-  
+  }
+
   if(scale.fix & is.null(init.phi)){
     stop("If scale.fix=TRUE, then init.phi must be supplied")
   }
-  
+
   ### First, get all the relevant elements from the arguments
   dat <- model.frame(formula, data, na.action=na.pass)
   nn <- dim(dat)[1]
-  
+
   if(typeof(data) == "environment"){
     id <- id
     weights <- weights
@@ -31,7 +31,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   }
   else{
     if(length(call$id) == 1){
-      subj.col <- which(colnames(data) == call$id)  
+      subj.col <- which(colnames(data) == call$id)
       if(length(subj.col) > 0){
         id <- data[,subj.col]
       }else{
@@ -40,9 +40,9 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     }else if(is.null(call$id)){
       id <- 1:nn
     }
-    
+
     if(length(call$weights) == 1){
-      weights.col <- which(colnames(data) == call$weights)  
+      weights.col <- which(colnames(data) == call$weights)
       if(length(weights.col) > 0){
         weights <- data[,weights.col]
       }else{
@@ -51,9 +51,9 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     }else if(is.null(call$weights)){
       weights <- rep.int(1,nn)
     }
-    
+
     if(length(call$waves) == 1){
-      waves.col <- which(colnames(data) == call$waves)  
+      waves.col <- which(colnames(data) == call$waves)
       if(length(waves.col) > 0){
         waves <- data[,waves.col]
       }else{
@@ -61,50 +61,53 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
       }
     }else if(is.null(call$waves)){
       waves <- NULL
-    }    
+    }
   }
   dat$id <- id
   dat$weights <- weights
   dat$waves <- waves
-  
+
   if(!is.numeric(dat$waves) & !is.null(dat$waves)) stop("waves must be either an integer vector or NULL")
-  
+
   # W is diagonal matrix of weights, sqrtW = sqrt(W)
   # included is diagonal matrix with 1 if weight > 0, 0 otherwise
   # includedvec is logical vector with T if weight > 0, F otherwise
-  # Note that we need to assign weight 0 to rows with NAs 
+  # Note that we need to assign weight 0 to rows with NAs
   # in order to preserve the correlation structure
   na.inds <- NULL
-  
+
   if(any(is.na(dat))){
     na.inds <- which(is.na(dat), arr.ind=T)
   }
-  
+
   #SORT THE DATA ACCORDING TO WAVES
   if(!is.null(waves)){
     dat <- dat[order(id, waves),]
+  }else{
+    dat <- dat[order(id),]
   }
-  
+
+
   # Figure out the correlation structure
   cor.vec <- c("independence", "ar1", "exchangeable", "m-dependent", "unstructured", "fixed", "userdefined")
   cor.match <- charmatch(corstr, cor.vec)
-  
-  if(is.na(cor.match)){stop("Unsupported correlation structure")}  
-  
+
+  if(is.na(cor.match)){stop("Unsupported correlation structure")}
+
   if(!is.null(dat$waves)){
     wavespl <- split(dat$waves, dat$id)
     idspl <- split(dat$id, dat$id)
-    
+
     maxwave <- rep(0, length(wavespl))
     incomp <- rep(0, length(wavespl))
-    
+
     for(i in 1:length(wavespl)){
       maxwave[i] <- max(wavespl[[i]]) - min(wavespl[[i]]) + 1
       if(maxwave[i] != length(wavespl[[i]])){
         incomp[i] <- 1
       }
     }
-  
+
     #If there are gaps and correlation isn't exchangeable or independent
     #then we'll add some dummy rows
     if(!is.element(cor.match, c(1,3)) & (sum(incomp) > 0) & !nodummy){
@@ -119,19 +122,19 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     weights[unique(na.inds[,1])] <- 0
     for(i in unique(na.inds)[,2]){
       if(is.factor(dat[,i])){
-        dat[na.inds[,1], i] <- levels(dat[,i])[1]  
+        dat[na.inds[,1], i] <- levels(dat[,i])[1]
       }else{
         dat[na.inds[,1], i] <- median(dat[,i], na.rm=T)
       }
     }
   }
-  
-  
+
+
   includedvec <- weights>0
-  
-  
+
+
   inclsplit <- split(includedvec, id)
-  
+
   dropid <- NULL
   allobs <- T
   if(any(!includedvec)){
@@ -139,43 +142,43 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     for(i in 1:length(unique(id))){
       if(all(!inclsplit[[i]])){
         dropid <- c(dropid, i)
-      }    
+      }
     }
   }
-  
-  
+
+
   if(length(dropid)>0){
     dropind <- which(is.element(id, dropid))
     dat <- dat[-dropind,]
     includedvec <- includedvec[-dropind]
     weights <- weights[-dropind]
-    
+
     id <- id[-dropind]
   }
   nn <- dim(dat)[1]
   K <- length(unique(id))
 
-  
+
   modterms <- terms(formula)
-  
+
   X <- model.matrix(formula,dat)
   Y <- model.response(dat)
-  offset <- model.offset(dat)			
-  
+  offset <- model.offset(dat)
+
   p <- dim(X)[2]
-  
-  
-  
+
+
+
   ### if no offset is given, then set to zero
   if(is.null(offset)){
     off <- rep(0, nn)
   }else{
     off <- offset
   }
-  
+
   # Is there an intercept column?
   interceptcol <- apply(X==1, 2, all)
-  
+
   ## Basic check to see if link and variance functions make any kind of sense
   linkOfMean <- LinkFun(mean(Y))
   if( any(is.infinite(linkOfMean) | is.nan(linkOfMean)) ){
@@ -184,7 +187,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   if( any(is.infinite( VarFun(mean(Y))) | is.nan( VarFun(mean(Y)))) ){
     stop("Infinite or NaN in the variance of the mean of responses.  Make sure variance function makes sense for these data.")
   }
-  
+
   if(is.null(init.beta)){
     if(any(interceptcol)){
       #if there is an intercept and no initial beta, then use link of mean of response
@@ -194,26 +197,26 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
       stop("Must supply an initial beta if not using an intercept.")
     }
   }
-  
-  
+
+
   # Number of included observations for each cluster
   includedlen <- rep(0, K)
   len <- rep(0,K)
   uniqueid <- unique(id)
-  
+
   tmpwgt <- as.numeric(includedvec)
   idspl <-ifelse(tmpwgt==0, NA, id)
   includedlen <- as.numeric(summary(split(Y, idspl, drop=T))[,1])
   len <- as.numeric(summary(split(Y, id, drop=T))[,1])
-  
+
   W <- Diagonal(x=weights)
   sqrtW <- sqrt(W)
   included <- Diagonal(x=(as.numeric(weights>0)))
-  
+
   # Get vector of cluster sizes... remember this len variable
   #len <- as.numeric(summary(split(Y, id, drop=T))[,1])
-  
-  
+
+
 
   # Set the initial alpha value
   if(is.null(init.alpha)){
@@ -229,7 +232,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
       alpha.new <- rep(0.2, max(unique(as.vector(corr.mat))))
     }
   }else{
-    alpha.new <- init.alpha	
+    alpha.new <- init.alpha
   }
   #if no initial overdispersion parameter, start at 1
   if(is.null(init.phi)){
@@ -237,17 +240,20 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   }else{
     phi <- init.phi
   }
-  
-  beta <- init.beta
-  
 
-  
+  beta <- init.beta
+
+
   #Set up matrix storage
   StdErr <- Diagonal(nn)
   dInvLinkdEta <- Diagonal(nn)
   Resid <- Diagonal(nn)
-  
-  
+#  if( (max(len)==1) & cor.match != 1 ){
+#    warning("Largest cluster size is 1. Changing working correlation to independence.")
+#    cor.match <- 1
+#    corstr <- "independence"
+#  }
+
   # Initialize for each correlation structure
   if(cor.match == 1){
     # INDEPENDENCE
@@ -266,14 +272,14 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     row.vec <- tmp$row.vec
     col.vec <- tmp$col.vec
     BlockDiag <- getBlockDiag(len)$BDiag
-    
+
   }else if(cor.match == 3){
     # EXCHANGEABLE
     # Build a block diagonal correlation matrix for updating and sandwich calculation
     # this matrix is block diagonal with all ones.  Each block is of dimension cluster size.
     tmp <- getBlockDiag(len)
     BlockDiag <- tmp$BDiag
-    
+
     # Create a vector of length number of observations with associated cluster size for each observation
     n.vec <- vector("numeric", nn)
     index <- c(cumsum(len) - len, nn)
@@ -285,9 +291,9 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     if(Mv >= max(len)){
       stop("Cannot estimate that many parameters: Mv >=  max(clustersize)")
     }
-    
+
     # Build block diagonal similar to in exchangeable case, also get row indices and column
-    # indices for fast matrix updating later.		
+    # indices for fast matrix updating later.
     tmp <- getBlockDiag(len)
     BlockDiag <- tmp$BDiag
     row.vec <- tmp$row.vec
@@ -301,18 +307,18 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     tmp <- getBlockDiag(len)
     BlockDiag <- tmp$BDiag
     row.vec <- tmp$row.vec
-    col.vec <- tmp$col.vec	
+    col.vec <- tmp$col.vec
   }else if(cor.match == 6){
     # FIXED
     # check if matrix meets some basic conditions
     corr.mat <- checkFixedMat(corr.mat, len)
-    
+
     R.alpha.inv <- as(getAlphaInvFixed(corr.mat, len), "symmetricMatrix")/phi
     BlockDiag <- getBlockDiag(len)$BDiag
   }else if(cor.match == 7){
     # USERDEFINED
     corr.mat <- checkUserMat(corr.mat, len)
-    
+
     # get the structure of the correlation matrix in a way that
     # I can use later on.
     tmp1 <- getUserStructure(corr.mat)
@@ -320,50 +326,64 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
     user.row <- tmp1$row.vec
     user.col <- tmp1$col.vec
     struct.vec <- tmp1$struct.vec
-    
+
     # the same block diagonal trick.
     tmp2 <- getBlockDiag(len)
     BlockDiag <- tmp2$BDiag
     row.vec <- tmp2$row.vec
     col.vec <- tmp2$col.vec
-    
+
   }else if(cor.match == 0){
     stop("Ambiguous Correlation Structure Specification")
   }else{
-    stop("Unsupported Correlation Structure")	
+    stop("Unsupported Correlation Structure")
   }
-  
+
   stop <- F
-  converged <- F	
+  converged <- F
   count <- 0
   beta.old <- beta
   unstable <- F
   phi.old <- phi
-  
-  
+
+
   # Main fisher scoring loop
-  while(!stop){		
+  while(!stop){
     count <- count+1
-    
+
     eta <- as.vector(X %*% beta) + off
-    
+
     mu <- InvLink(eta)
-    
+
     diag(StdErr) <- sqrt(1/VarFun(mu))
-    
+
+    ###### REMOVE LATER
+    n.vec <- vector("numeric", nn)
+    index <- c(cumsum(len) - len, nn)
+    for(i in 1:K){
+      n.vec[(index[i]+1) : index[i+1]] <-  rep(len[i], len[i])
+    }
+    R.alpha.inv <- getAlphaInvEX(0.2, n.vec, BlockDiag)
+    beta.list <- updateBeta(Y, X, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, sqrtW)
+    beta <- beta.list$beta
+    eta <- as.vector(X %*% beta) + off
+    mu <- InvLink(eta)
+    diag(StdErr) <- sqrt(1/VarFun(mu))
+    ###########
     if(!scale.fix){
       phi <- updatePhi(Y, mu, VarFun, p, StdErr, included, includedlen, sqrtW)
     }
     phi.new <- phi
-    
-    
+
+
+
     ## Calculate alpha, R(alpha)^(-1) / phi
     if(cor.match == 2){
       # AR-1
       alpha.new <- updateAlphaAR(Y, mu, VarFun, phi, id, len, StdErr, p, included, includedlen, includedvec, allobs, sqrtW, BlockDiag)
       R.alpha.inv <- getAlphaInvAR(alpha.new, a1,a2,a3,a4, row.vec, col.vec)/phi
     }else if(cor.match == 3){
-      #EXCHANGEABLE		  
+      #EXCHANGEABLE
       alpha.new <- updateAlphaEX(Y, mu, VarFun, phi, id, len, StdErr, Resid, p, BlockDiag, included, includedlen, sqrtW)
       R.alpha.inv <- getAlphaInvEX(alpha.new, n.vec, BlockDiag)/phi
     }else if(cor.match == 4){
@@ -380,7 +400,7 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
         stop <- T
         warning("some estimated correlation is greater than 1, stopping.")
       }
-      R.alpha.inv <- getAlphaInvMDEP(alpha.new, len, row.vec, col.vec)/phi		
+      R.alpha.inv <- getAlphaInvMDEP(alpha.new, len, row.vec, col.vec)/phi
     }else if(cor.match == 5){
       # UNSTRUCTURED
       alpha.new <- updateAlphaUnstruc(Y, mu, VarFun, phi, id, len, StdErr, Resid,  p, BlockDiag, included, includedlen, allobs, sqrtW)
@@ -402,50 +422,51 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
       R.alpha.inv <-  Diagonal(x = rep.int(1/phi, nn))
       alpha.new <- "independent"
     }
-    
-    
-    beta.list <- updateBeta(Y, X, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, sqrtW)	
+
+
+    beta.list <- updateBeta(Y, X, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, W)
     beta <- beta.list$beta
     phi.old <- phi
     if( max(abs((beta - beta.old)/(beta.old + .Machine$double.eps))) < tol ){converged <- T; stop <- T}
-    if(count >= maxit){stop <- T}		
-    beta.old <- beta		
+    if(count >= maxit){stop <- T}
+    beta.old <- beta
   }
   biggest <- which.max(len)[1]
-  index <- cumsum(len[biggest])
+  index <- sum(len[1:biggest])-len[biggest]
+  
   if(K == 1){
     biggest.R.alpha.inv <- R.alpha.inv
     if(cor.match == 6) {
-      biggest.R.alpha <- corr.mat
+      biggest.R.alpha <- corr.mat*phi
     }else{
       biggest.R.alpha <- solve(R.alpha.inv)
     }
   }else{
     biggest.R.alpha.inv <- R.alpha.inv[(index+1):(index+len[biggest]) , (index+1):(index+len[biggest])]
     if(cor.match == 6){
-      biggest.R.alpha <- corr.mat[(index+1):(index+len[biggest]) , (index+1):(index+len[biggest])]
+      biggest.R.alpha <- corr.mat[1:len[biggest] , 1:len[biggest]]*phi
     }else{
       biggest.R.alpha <- solve(biggest.R.alpha.inv)
     }
   }
 
-  
+
   eta <- as.vector(X %*% beta) + off
   if(sandwich){
-    sandvar.list <- getSandwich(Y, X, eta, id, R.alpha.inv, phi, InvLinkDeriv, InvLink, VarFun, beta.list$hess, StdErr, dInvLinkdEta, BlockDiag, sqrtW)
+    sandvar.list <- getSandwich(Y, X, eta, id, R.alpha.inv, phi, InvLinkDeriv, InvLink, VarFun, beta.list$hess, StdErr, dInvLinkdEta, BlockDiag, W)
   }else{
     sandvar.list <- list()
     sandvar.list$sandvar <- "no sandwich"
   }
-  
+
   if(!converged){warning("Did not converge")}
   if(unstable){warning("Number of subjects with number of observations >= Mv is very small, some correlations are estimated with very low sample size.")}
-  
-  
+
+
   # Create object of class geem with information about the fit
   dat <- model.frame(formula, data, na.action=na.pass)
   X <- model.matrix(formula, dat)
-  
+
   if(is.character(alpha.new)){alpha.new <- 0}
   results <- list()
   results$beta <- as.vector(beta)
@@ -468,8 +489,9 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
   results$eta <- eta
   results$dropped <- dropid
   results$weights <- weights
+  results$terms <- modterms
   results$y <- Y
-  results$biggest.R.alpha <- biggest.R.alpha
+  results$biggest.R.alpha <- biggest.R.alpha/phi
   class(results) <- "geem"
   return(results)
 }
@@ -483,47 +505,49 @@ geem <- function(formula, id, waves=NULL, data = parent.frame(), family = gaussi
 updatePhi <- function(YY, mu, VarFun, p, StdErr, included, includedlen, sqrtW){
   nn <- sum(includedlen)
   resid <- diag(StdErr %*% included %*% sqrtW %*% Diagonal(x = YY - mu))
-  phi <- (1/(sum(included)-p))*crossprod(resid, resid) 
-  return(as.numeric(phi))	
+  phi <- (1/(sum(included)-p))*crossprod(resid, resid)
+
+  return(as.numeric(phi))
 }
 
 ### Method to update coefficients.  Goes to a maximum of 10 iterations, or when
 ### rough convergence has been obtained.
-updateBeta = function(YY, XX, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, sqrtW){
+updateBeta = function(YY, XX, beta, off, InvLinkDeriv, InvLink, VarFun, R.alpha.inv, StdErr, dInvLinkdEta, tol, W){
   beta.new <- beta
   conv=F
   for(i in 1:10){
     eta <- as.vector(XX%*%beta.new) + off
-    
+
     diag(dInvLinkdEta) <- InvLinkDeriv(eta)
-    mu <- InvLink(eta)	
+    mu <- InvLink(eta)
     diag(StdErr) <- sqrt(1/VarFun(mu))
-    
-    hess <- crossprod(sqrtW %*% StdErr %*% dInvLinkdEta %*%XX, R.alpha.inv %*% sqrtW %*% StdErr %*%dInvLinkdEta %*% XX)
-    esteq <- crossprod(sqrtW %*% StdErr %*%dInvLinkdEta %*%XX , R.alpha.inv %*% sqrtW %*% StdErr %*% (YY - mu))
-    
+
+    hess <- crossprod(  StdErr %*% dInvLinkdEta %*%XX, R.alpha.inv %*% W %*% StdErr %*%dInvLinkdEta %*% XX)
+    esteq <- crossprod(   StdErr %*%dInvLinkdEta %*%XX , R.alpha.inv %*% W %*% StdErr %*% as.matrix(YY - mu)) 
+
     update <- solve(hess, esteq)
     if(max(abs(update)/beta.new) < 100*tol){break}
-    
+
     beta.new <- beta.new + as.vector(update)
+
   }
   return(list(beta = beta.new, hess = hess))
 }
 
 ### Calculate the sandiwch estimator as usual.
-getSandwich = function(YY, XX, eta, id, R.alpha.inv, phi, InvLinkDeriv, InvLink, VarFun, hessMat, StdErr, dInvLinkdEta, BlockDiag, sqrtW){
-  
+getSandwich = function(YY, XX, eta, id, R.alpha.inv, phi, InvLinkDeriv, InvLink, VarFun, hessMat, StdErr, dInvLinkdEta, BlockDiag, W){
+
   diag(dInvLinkdEta) <- InvLinkDeriv(eta)
-  mu <- InvLink(eta)			
+  mu <- InvLink(eta)
   diag(StdErr) <- sqrt(1/VarFun(mu))
   scoreDiag <- Diagonal(x= YY - mu)
   BlockDiag <- scoreDiag %*% BlockDiag %*% scoreDiag
-  
-  numsand <- crossprod(sqrtW %*% StdErr %*% dInvLinkdEta %*% XX, R.alpha.inv %*% sqrtW %*% StdErr %*% BlockDiag %*% StdErr %*% sqrtW %*% R.alpha.inv %*% sqrtW %*% StdErr %*% dInvLinkdEta %*% XX)
+
+  numsand <- as.matrix(crossprod(  StdErr %*% dInvLinkdEta %*% XX, R.alpha.inv %*% W %*% StdErr %*% BlockDiag %*% StdErr %*% W %*% R.alpha.inv %*%  StdErr %*% dInvLinkdEta %*% XX))
   
   sandvar <- t(solve(hessMat, numsand))
   sandvar <- t(solve(t(hessMat), sandvar))
-  
+
   return(list(sandvar = sandvar, numsand = numsand))
 }
 
